package cn.com.infosec.crypto.engines;

import cn.com.infosec.crypto.BlockCipher;
import cn.com.infosec.crypto.CipherParameters;
import cn.com.infosec.crypto.DataLengthException;
import cn.com.infosec.crypto.params.KeyParameter;

public class RC6Engine implements BlockCipher {
   private static final int wordSize = 32;
   private static final int bytesPerWord = 4;
   private static final int _noRounds = 20;
   private static final int P32 = -1209970333;
   private static final int Q32 = -1640531527;
   private static final int LGW = 5;
   private int[] _S = null;
   private boolean forEncryption;

   private void setKey(byte[] param1) {
      // $FF: Couldn't be decompiled
   }

   private int encryptBlock(byte[] in, int inOff, byte[] out, int outOff) {
      int var10002 = inOff;
      int var10005 = inOff;
      int in1 = this.bytesToWord(in, inOff);
      inOff = this.bytesToWord(in, inOff + 4);
      int var5 = this.bytesToWord(in, var10005 + 8);
      int var10000 = this.bytesToWord(in, var10002 + 12);
      int[] var13 = this._S;
      inOff += var13[0];
      int var6 = var10000 + var13[1];

      for(int var7 = 1; var7 <= 20; inOff = var10000) {
         int var14 = var5;
         int var10003 = in1;
         in1 = this.rotateLeft(inOff * (inOff * 2 + 1), 5);
         var5 = this.rotateLeft(var6 * (var6 * 2 + 1), 5);
         int var8;
         int var9 = this.rotateLeft(var10003 ^ in1, var5) + this._S[var8 = var7 * 2];
         in1 = this.rotateLeft(var14 ^ var5, in1) + this._S[var8 + 1];
         ++var7;
         var10000 = in1;
         var5 = var6;
         in1 = inOff;
         var6 = var9;
      }

      int[] var18 = this._S;
      int var10 = in1 + var18[42];
      in1 = var5 + var18[43];
      this.wordToBytes(var10, out, outOff);
      this.wordToBytes(inOff, out, outOff + 4);
      this.wordToBytes(in1, out, outOff + 8);
      this.wordToBytes(var6, out, outOff + 12);
      return 16;
   }

   private int decryptBlock(byte[] in, int inOff, byte[] out, int outOff) {
      int var10000 = this.bytesToWord(in, inOff);
      int var10004 = inOff;
      int in1 = this.bytesToWord(in, inOff + 4);
      inOff = this.bytesToWord(in, inOff + 8);
      int var5 = this.bytesToWord(in, var10004 + 12);
      int[] var10001 = this._S;
      inOff -= var10001[43];
      int var6 = var10000 - var10001[42];

      for(int var7 = 20; var7 >= 1; var5 = var10000) {
         int var13 = var5;
         var10004 = in1;
         in1 = this.rotateLeft(var6 * (var6 * 2 + 1), 5);
         var5 = this.rotateLeft(inOff * (inOff * 2 + 1), 5);
         int var8;
         int var9 = this.rotateRight(var10004 - this._S[(var8 = var7 * 2) + 1], in1) ^ var5;
         in1 ^= this.rotateRight(var13 - this._S[var8], var5);
         --var7;
         var10000 = inOff;
         int var10002 = var6;
         var6 = in1;
         in1 = var10002;
         inOff = var9;
      }

      int[] var10007 = this._S;
      int var10 = var5 - var10007[1];
      in1 -= var10007[0];
      this.wordToBytes(var6, out, outOff);
      this.wordToBytes(in1, out, outOff + 4);
      this.wordToBytes(inOff, out, outOff + 8);
      this.wordToBytes(var10, out, outOff + 12);
      return 16;
   }

   private int rotateLeft(int x, int y) {
      return x << y | x >>> -y;
   }

   private int rotateRight(int x, int y) {
      return x >>> y | x << -y;
   }

   private int bytesToWord(byte[] src, int srcOff) {
      int var4 = 0;

      for(int var3 = 3; var3 >= 0; --var3) {
         var4 = (var4 << 8) + (src[var3 + srcOff] & 255);
      }

      return var4;
   }

   private void wordToBytes(int word, byte[] dst, int dstOff) {
      for(int var4 = 0; var4 < 4; ++var4) {
         int var10000 = word;
         int var10002 = word;
         word = var4 + dstOff;
         dst[word] = (byte)var10002;
         word = var10000 >>> 8;
      }

   }

   public String getAlgorithmName() {
      return "RC6";
   }

   public int getBlockSize() {
      return 16;
   }

   public void init(boolean forEncryption, CipherParameters params) {
      if (params instanceof KeyParameter) {
         RC6Engine var10000 = this;
         RC6Engine var10001 = this;
         KeyParameter var3 = (KeyParameter)params;
         var10001.forEncryption = forEncryption;
         var10000.setKey(var3.getKey());
      } else {
         throw new IllegalArgumentException("invalid parameter passed to RC6 init - " + params.getClass().getName());
      }
   }

   public int processBlock(byte[] in, int inOff, byte[] out, int outOff) {
      int var5 = this.getBlockSize();
      if (this._S != null) {
         if (inOff + var5 <= in.length) {
            if (outOff + var5 <= out.length) {
               return this.forEncryption ? this.encryptBlock(in, inOff, out, outOff) : this.decryptBlock(in, inOff, out, outOff);
            } else {
               throw new DataLengthException("output buffer too short");
            }
         } else {
            throw new DataLengthException("input buffer too short");
         }
      } else {
         throw new IllegalStateException("RC6 engine not initialised");
      }
   }

   public void reset() {
   }
}
